function [obj, X, Y, f] = GMgen
% randomly generate a Guassian mixutre model
% returns the GM object as input for kHerding_resample_GM

K = 20;        % # mixture components
D = 2;          % # dimensions

stdmu = 4;     % parameter for mean
stdsig = 2;     % parameter for covariance matrix
ratio = [.1,2]; % parameter for covariance matrix
al = K;         % parameter for pi

% mean
MU = (2*rand(K,D)-1)*stdmu;

% covariance matrix
SIG = zeros(D,D,K);
for k=1:K
  sig = (rand(D,1)*(ratio(2)-ratio(1))+ratio(1))*stdsig;
  SIG(:,:,k) = diag(sig.^(-2)); % sig: .1~2
  [U,R]=qr(randn(D));
  SIG(:,:,k) = U*SIG(:,:,k)*U';
end

% mixture probability, pi
p = randg(al,[1,K]); p = p/sum(p);

obj = gmdistribution(MU,SIG,p);

if D==2
  [X,Y]=meshgrid(min(obj.mu(:,1))-2:.1:max(obj.mu(:,1))+2,...
    min(obj.mu(:,2))-2:.1:max(obj.mu(:,2))+2);
  f = pdf(obj,[X(:),Y(:)]);
  f = reshape(f,size(X));
  figure
  contour(X,Y,f,20)
  hold on
  plot(obj.mu(:,1),obj.mu(:,2),'or');
  hold off
else
  X = [];
  Y = [];
  f = [];
end
